# redcayBFSdemo.py

'''DESCRIPTION: this program demonstrates the functions of the getDirections
screen using a simple 2-D object oriented graphics library.'''

from graphics import *
from button import *
from redcaynodes import *
import copy

#------------------------------------------------------------------------------

def getDirectionsPrompt():
    '''Displays a prompt on the window to collect room numbers.'''
    
    dirPrompt = Rectangle(Point(640, 270), Point(1250, 580))
    dirPrompt.setFill("white")
    dirPrompt.setOutline("blue")
    dirPrompt.setWidth(18)
    dirPrompt.draw(win)

    # display imported images
    redcayExt = Image(Point(285, 430), "redcayExterior.gif")
    redcayExt.draw(win)
    card = Image(Point(1290, 500), "plattsCardinalLogo3.gif")
    card.draw(win)
    
    enterRoomText = Text(Point(900, 200), "Enter the Room Numbers")
    enterRoomText.setSize(32)
    enterRoomText.draw(win)
    roomFromText = Text(Point(800, 325), "FROM     Room # :")
    roomToText = Text(Point(800, 400),   "TO       Room # :")
    roomFromText.setSize(22)
    roomToText.setSize(22)
    roomFromText.draw(win)
    roomToText.draw(win)

    # display user input fields
    roomFromEntry = Entry(Point(1000, 325), 5)
    roomToEntry = Entry(Point(1000, 400), 5)
    roomFromEntry.setSize(22)
    roomToEntry.setSize(22)
    roomFromEntry.draw(win)
    roomToEntry.draw(win)

    # simulate a sample path run.
    dirScreenSamplePath()

    # activate all valid buttons
    getDirButton = Button(win, Point(785, 525), 250, 50, "Get Directions")
    getDirButton.activate()
    showMapButton = Button(win, Point(1075, 525), 250, 50, "Show Map")
    showMapButton.activate()
    needHelpButton = Button(win, Point(1175, 300), 100, 30, "Need Help?")
    needHelpButton.activate()

    # wait for a valid click
    while True:
        click = win.getMouse()
        
        if needHelpButton.clicked(click):
            needHelpButton.deactivate()
            dislayErrorMessage()
            needHelpButton.activate()
            
        elif getDirButton.clicked(click):
            getDirButton.deactivate()

            # convert both room inputs to uppercase
            roomFrom = (roomFromEntry.getText().upper())
            roomTo = (roomToEntry.getText().upper())

            if debug:
                print("--USER INPUT--\n\nRoomFrom ---> ", roomFrom)
                print("\nRoomTo   ---> ", roomTo, "\n")
            
            # invalidates empty strings for user input                                  
            if (roomFrom == '') or (roomTo == ''):
                print("Empty String Error\n")
                dislayErrorMessage()
                getDirButton.activate()
                
            # checks if input room numbers are valid, returns their node pairing
            startNode =  validRoom(roomFrom)
            endNode = validRoom(roomTo)

            if startNode is not None and endNode is not None:
                getDirButton.deactivate()
                roomFromEntry.undraw()
                roomToEntry.undraw()
                print("VALID INPUT.\n")
                break
            else:
                dislayErrorMessage()
                getDirButton.activate()
                print("INVALID ROOM INPUT. Type two VALID ROOM numbers.\n\n\n")
                printValidRooms()
            
        elif showMapButton.clicked(click):
            showMapButton.deactivate()      
            roomFromEntry.undraw()
            roomToEntry.undraw()
            startNode = None
            endNode = None
            break
        else:
            continue

    return startNode, endNode
        

def dislayErrorMessage():
    '''Display a universal error message for invalid input'''
    
    errorText = Text(Point(930, 450), "Enter valid room numbers.    ex.) 201")  
    errorText.setFill("red")
    errorText.draw(win)
    time.sleep(2)
    errorText.undraw()

    
def dirScreenSamplePath():
    '''Illustrates a hardcoded sample path traversal on the Directions screen.'''
    
    # points for the sample path on the getDirections screen
    p1 = Point(285, 680)
    p2 = Point(45, 680)
    p3 = Point(45, 200)
    p4 = Point(550, 200)
    p5 = Point(550, 680)
    p6 = Point(950, 680)
    p7 = Point(1400, 680)
    
    demopath = [p1, p2, p3, p4, p5, p6, p7]

    # make the starting point flicker
    startnodeIndicator(p1, 25)

    # store the length of the demopath list
    demopathlistlength = len(demopath)

    # iterate through each node and place a line between them successively
    for i in range(demopathlistlength-1):
        currpoint = demopath.pop(0)
        node = Circle(currpoint, 20)
        node.setFill("green")
        node.setOutline("green")
        node.draw(win)
        nextpoint = demopath.pop(0)

        # draw an intermediate line
        drawMidLine(currpoint, nextpoint, 15)
        
        demopath.insert(0, nextpoint) 
        time.sleep(0.25)

        # on the last iteration,  indicate the path's terminating node
        if i == (demopathlistlength-2):
            
            # toggle the final node's colors
            toggleNodeColors(nextpoint, 5, 25, "black")


def startnodeIndicator(point, radius):
    '''Toggle the colors of the starting node three times to indicate
the starting point of the route in the graphic window.'''
    
    for i in range(3):
        startnode = Circle(point, radius)
        startnode.setFill("green")
        startnode.draw(win)
        time.sleep(0.25)
        startnode.undraw()
        startnode.setFill("blue")
        startnode.draw(win)
        time.sleep(0.25)
        startnode.undraw()
    

def toggleNodeColors(point, width, radius, final_outline_color):
    '''Toggle the colors of a node five times alternating green and red.'''

    for i in range(3):
        node = Circle(point, radius)
        node.setWidth(width)
        node.setFill("green")
        node.draw(win)
        time.sleep(0.25) 
        node.setFill("red")
        node.undraw()
        time.sleep(0.25)
        node.setWidth(width)
        node.setOutline(final_outline_color)
        node.draw(win)


def drawMidLine(point1, point2, linewidth):
    '''display one line from a selected point to a computed midpoint and another
line from the computed midpoint to another point (for animation purposes).'''  

    # get midpoint coordinates
    midx = abs(point2.getX() + point1.getX())/2
    midy = abs(point2.getY() + point1.getY())/2

    # plot line from currpoint to midpoint
    line1 = Line(point1, Point(midx, midy))
    line1.setWidth(linewidth)
    line1.setFill("green")
    line1.draw(win)
    time.sleep(0.25) #.5
    
    # plot line from midpoint to nextpoint
    line2 = Line(Point(midx, midy), point2)
    line2.setWidth(linewidth)
    line2.setFill("green")
    line2.draw(win)


def moreDirectionsOption():
    moreDirButton = Button(win, Point(700, 25), 200, 40, "GET DIRECTIONS")
    moreDirButton.activate()
    exitButton = Button(win, Point(1000, 25), 100, 40, "EXIT")
    exitButton.activate()
    while True:
            click = win.getMouse()
            if exitButton.clicked(click):
                exitButton.deactivate()
                moreDirButton.deactivate()
                if debug:
                    print("Exit Button clicked correctly.\n")
                continueLoop = False
                break
            if moreDirButton.clicked(click):
                moreDirButton.deactivate()
                if debug:
                    print("moreDirButton clicked correctly.\n")
                continueLoop = True
                break
            
    return continueLoop


def drawRoute(path):
    '''Display the shortest path between the two rooms chosen by the user.'''

    route = path

    # store the length of the route
    routelength = len(route)
    
    #num_of_edges = len(path)-1

    startingnodeString = route.pop(0)    # grab the first node in the path. 

    # toggle the starting node indicator.
    startnode = eval(startingnodeString)
    startnodeIndicator(startnode, 12)

    route.insert(0, startingnodeString) # place the first node back in the path.
  
    for i in range(routelength-1):
        currpointString = route.pop(0)
        currpoint = eval(currpointString)
        node = Circle(currpoint, 10)
        node.setFill("green")
        node.setOutline("green")
        node.draw(win)
        nextpointString = route.pop(0)
        nextpoint = eval(nextpointString)

        drawMidLine(currpoint, nextpoint, 7)

        route.insert(0, nextpointString) 
        time.sleep(0.25)

        # on the last iteration,  indicate the path's terminating node
        if i == (routelength-2): 
            
            # toggle the final node's colors
            toggleNodeColors(nextpoint, 4, 12, "red")
    

#==============================================================================
        
if __name__ == '__main__':
    
    debug = False     # DEBUG MODE is initialized to OFF by default.

    # check for DEBUG MODE command-line invocation ("-d" or "-D").
    for arg in sys.argv:
        if (arg == "-d") or (arg == "-D"):
            debug = True
            print("\n================= DEBUG MODE ====================\n\n")
            
    SCREEN_WIDTH = 1700
    SCREEN_HEIGHT = 800
    win = GraphWin("Directions", SCREEN_WIDTH, SCREEN_HEIGHT)

    print("PROGRAM STARTED...\n\n")
    printValidRooms()
    
    continueLoop = True
    while(continueLoop):
    
        startnode, endnode = getDirectionsPrompt()

        pic = Image(Point(910, 415), "RedcayScan_2ndFloor.gif")    
        pic.draw(win)

        if startnode is None or endnode is None:
            # is true when "Show Map" button is clicked.

            if debug:
                plotNodes(win)
                
            # ask user if they decided to enter directions. 
            continueLoop = moreDirectionsOption()
        elif startnode == endnode:
            print("The room numbers are the same.")
            print("The user is at the destination.\n")
                
            toggleNodeColors(eval(endnode), 4, 10, "red")

            if debug:
                plotNodes(win)
            continueLoop = moreDirectionsOption()            
        else:
            # Room numbers are valid and not equal --> find the shortest route.
            path = shortest_path(graph, startnode, endnode)
            shortpath = copy.deepcopy(path)

            drawRoute(path)
            
            if debug:
                print("Start Node: ", startnode, "\n")
                print("End   Node: ", endnode, "\n")
                print("Shortest Path: ", shortpath)
                print("\n\n\ncontinueLoop = ", continueLoop, "\n")
                plotNodes(win) 

            # ask the user if they would like to get more directions.
            continueLoop = moreDirectionsOption()

        if continueLoop:
            pic.undraw()
            resetScreen = Rectangle(Point(0,0), \
                                    Point(SCREEN_WIDTH, SCREEN_HEIGHT))
            resetScreen.setFill("grey90")
            resetScreen.draw(win)
            printValidRooms()

    if debug:
        testDisplay = Text(Point(150, 780), \
                           "Click anywhere to display coordinates.")
        testDisplay.draw(win)
        doneButton = Button(win, Point(1400, 775), 70, 40, "Done")
        doneButton.activate()
        
        while True:
            click = win.getMouse()
            clickinfo = "You clicked at:   x = {0}   y = {1}" \
                        .format(click.getX(), click.getY())
            print ("{0}\n".format(clickinfo))  # print coordinates to stdout.
            testDisplay.setText(clickinfo)
            if doneButton.clicked(click):
                doneButton.deactivate()
                break
  
    win.close()
    print("PROGRAM EXIT.\n")
