# redcaynodes.py

from graphics import *
from collections import deque


############################################################################

# these two functions taken from: http://code.activestate.com/recipes/576675-bfs-breadth-first-search-graph-traversal/
def bfs(g, start):
    queue, enqueued = deque([(None, start)]), set([start])
    while queue:
        parent, n = queue.popleft()
        yield parent, n
        new = set(g[n]) - enqueued
        enqueued |= new
        queue.extend([(n, child) for child in new])

def shortest_path(g, start, end):
    parents = {}
    for parent, child in bfs(g, start):
        parents[child] = parent
        if child == end:
            revpath = [end]
            while True:
                parent = parents[child]
                revpath.append(parent)
                if parent == start:
                    break
                child = parent
            return list(reversed(revpath))
    return None # or raise appropriate exception

############################################################################


# hard-coded Redcay 2nd Floor node mappings

n0  = Point(296, 564)             
n1  = Point(230, 564)       # Room 201
n2  = Point(296, 338)            
n3  = Point(575, 338)            
n4  = Point(805, 338)            
n5  = Point(805, 250)       # Room 229
n6  = Point(805, 500)       # Room 214
n7  = Point(975, 338)            
n8  = Point(975, 210)            
n9  = Point(975, 150)       # Room 231
n10 = Point(1050, 210)           
n11 = Point(1050, 150)      # Room 233
n12 = Point(1135, 210)          
n13 = Point(1135, 150)      # Room 235
n14 = Point(1220, 210)          
n15 = Point(1220, 150)      # Room 237
n16 = Point(1360, 210)           
n17 = Point(1450, 210)           
n18 = Point(1450, 150)      # Room 243   
n19 = Point(1450, 400)      # Room 236
n20 = Point(1510, 210)      # Room 245     
n21 = Point(1360, 310)          
n22 = Point(1360, 400)          
n23 = Point(1360, 485)          
n24 = Point(1450, 485)      # Room 234
n25 = Point(1360, 560)          
n26 = Point(1360, 640)      # Room 228
n27 = Point(1450, 310)      # Room 247
n28 = Point(1450, 560)          
n29 = Point(1450, 640)      # Room 230
n30 = Point(1510, 560)      # Room 232
n31 = Point(1300, 560)           
n32 = Point(1300, 640)      # Room 226
n33 = Point(1210, 560)          
n34 = Point(1210, 640)      # Room 224
n35 = Point(1150, 560)          
n36 = Point(1150, 640)      # Room 222
n37 = Point(1060, 560)          
n38 = Point(1060, 640)      # Room 220
n39 = Point(975, 560)            
n40 = Point(975, 640)       # Room 218
n41 = Point(1150, 338)
n42 = Point(1300, 338)


allNodes = [n0, n1, n2, n3, n4, n5, n6, n7, n8, n9, n10, n11,
            n12, n13, n14, n15, n16, n17, n18, n19, n20, n21,
            n22, n23, n24, n25, n26, n27, n28, n29, n30, n31,
            n32, n33, n34, n35, n36, n37, n38, n39, n40, n41,
            n42]

total_num_of_Nodes = len(allNodes) 

#============================================================================

def plotNodes(win):
    '''Plot every mapped node onto the graphic window'''

    for i in range(total_num_of_Nodes):
        point = allNodes[i] 
        node = Circle(point, 10)
        node.setFill("blue2")
        node.draw(win)
        label = Text(point, str(i))
        label.setFill("white")
        label.draw(win)

##def validRoom(graph, roomNum):
##    return roomNum in graph

def validRoom(inputRoom):
    '''check input rooms and return their corresponding nodes'''
    
    if inputRoom == '201':
        roomNode = 'n1'
    elif inputRoom == '229':
        roomNode = 'n5'
    elif inputRoom == '214':
        roomNode = 'n6'
    elif inputRoom == '231':
        roomNode = 'n9'
    elif inputRoom == '233':
        roomNode = 'n11'
    elif inputRoom == '235':
        roomNode = 'n13'
    elif inputRoom == '237':
        roomNode = 'n15'
    elif inputRoom == '243':
        roomNode = 'n18'
    elif inputRoom == '236':
        roomNode = 'n19'
    elif inputRoom == '245':
        roomNode = 'n20'
    elif inputRoom == '234':
        roomNode = 'n24'
    elif inputRoom == '228':
        roomNode = 'n26'
    elif inputRoom == '247':
        roomNode = 'n27'
    elif inputRoom == '230':
        roomNode = 'n29'
    elif inputRoom == '232':
        roomNode = 'n30'
    elif inputRoom == '226':
        roomNode = 'n32'
    elif inputRoom == '224':
        roomNode = 'n34'
    elif inputRoom == '222':
        roomNode = 'n36'
    elif inputRoom == '220':
        roomNode = 'n38'
    elif inputRoom == '218':
        roomNode = 'n40'  
    else:
        roomNode = None

    return roomNode


# a list of valid room numbers the user may input for getting directions.
# as of Date: 3/28/13
validroomlist = [201, 214, 218, 220, 222, 224, 226, 228, 229, 230,
                 231, 232, 233, 234, 235, 236, 237, 243, 245, 247 ]

def printValidRooms():
    print("------- VALID ROOMS: ------------------------------------------\n")
    print(validroomlist, "\n")
    print("---------------------------------------------------------------\n")

#-----------------------------------------------------------------------------

# adjacency list representation of all nodes
graph = {'n0' : ['n1', 'n2'],
         'n1' : ['n0'],
         'n2' : ['n0', 'n3'],
         'n3' : ['n2', 'n4'],
         'n4' : ['n3', 'n5', 'n6', 'n7'],
         'n5' : ['n4'],
         'n6' : ['n4'],
         'n7' : ['n4', 'n8', 'n41'],
         'n8' : ['n7', 'n9', 'n10'],
         'n9' : ['n8'],
         'n10': ['n8', 'n11', 'n12'],
         'n11': ['n10'],
         'n12': ['n10', 'n13', 'n14'],
         'n13': ['n12'],
         'n14': ['n12', 'n15', 'n16'],
         'n15': ['n14'],
         'n16': ['n14', 'n17', 'n21'],
         'n17': ['n16', 'n18', 'n20'],
         'n18': ['n17'],
         'n19': ['n22'],
         'n20': ['n17'],
         'n21': ['n16', 'n22', 'n27', 'n42'],
         'n22': ['n21', 'n19', 'n23', 'n42'],
         'n23': ['n22', 'n24', 'n25'],
         'n24': ['n23'],
         'n25': ['n23', 'n26', 'n28', 'n31'],
         'n26': ['n25'],
         'n27': ['n21'],
         'n28': ['n25', 'n29', 'n30'],
         'n29': ['n28'],
         'n30': ['n28'],
         'n31': ['n25', 'n32', 'n33'],
         'n32': ['n31'],
         'n33': ['n31', 'n34', 'n35'],
         'n34': ['n33'],
         'n35': ['n33', 'n36', 'n37'],
         'n36': ['n35'],
         'n37': ['n35', 'n38', 'n39'],
         'n38': ['n37'],
         'n39': ['n37', 'n40'],
         'n40': ['n39'],
         'n41': ['n7', 'n35', 'n42'],
         'n42': ['n21', 'n22', 'n41']}  


#=============================================================================



